#include <EEPROM.h>
// I2Cdev and MPU6050 must be installed as libraries
#include "I2Cdev.h"
#include "MPU6050.h"
#include "Wire.h"
#include "LedOnOff.h"

#define btnCalPin   4
#define btnBTOffPin 6
#define btnBTOffSig 5
#define buzPin      10

int sampleTimes=1000;     //Amount of readings used to average, make it higher to get more precision but sketch will be slower  (default:1000)
int sampleOffset=100;
int acel_deadzone=8;     //Acelerometer error allowed, make it lower to get more precision, but sketch may not converge  (default:8)
int giro_deadzone=1;     //Giro error allowed, make it lower to get more precision, but sketch may not converge  (default:1)

const long btSpeed = 38400;

MPU6050 accelgyro(0x68); // <-- use for AD0 high
LedOnOff ll = LedOnOff('L', 14, 500, buzPin, 2000);
LedOnOff lr = LedOnOff('R', 16, 500, buzPin, 2100);

typedef struct gyroval{
  int16_t ax;
  int16_t ay;
  int16_t az;
  int16_t gx;
  int16_t gy;
  int16_t gz;
  int8_t  btoff;
} GyroVal;

GyroVal gv;
int16_t gyValUD, gyValLR;

const int8_t OFF_d   = 0;
const int8_t LEFT_d  = 1;
const int8_t RIGHT_d = 2;

const int8_t LAST_d = 3;

const int16_t MIN_d        = -15000;
const int16_t MAX_d        = 15000;
const int16_t MAXINTTIME_d    = 1600;
const int16_t MAXSIGTIME_d = 400;

int8_t  ledState = OFF_d;
int8_t  oledState = OFF_d;
int8_t  led = OFF_d;

int8_t prevSignalLR = OFF_d;
int8_t stateLR  = 0;

int8_t signalUD = OFF_d;
int8_t numSigUD  = 0;

unsigned long startTime = millis();
unsigned long currTime = startTime;

int addr = 0;
int btnCalState = 0;

int8_t blueToothOff = 0;


//const byte numChars = 5;
//char rcvData[numChars];
//boolean newData = false;

//void checkSerial() {
//    static byte ndx = 0;
//    char endMarker = '#';
//    char rc;
//    
//    newData = false;
//
//    while (Serial1.available() > 0 && newData == false) {
//        rc = Serial1.read();
//        if (rc != endMarker) {
//            rcvData[ndx] = rc;
//            ndx++;
//            if (ndx >= numChars) {
//                ndx = numChars - 1;
//            }
//        }
//        else {
//            rcvData[ndx] = '\0'; // terminate the string
//            ndx = 0;
//            newData = true;
//        }
//    }
//}

//void checkSignal(int8_t a_sig) {
//    if (prevSignalLR != a_sig) {
//        if (stateLR == 0 && prevSignalLR == OFF_d) {
//            ledState = a_sig;
//            led = a_sig;
//            startTime = currTime;
//        }
//        stateLR++;
//        prevSignalLR = a_sig;
//    }
//}

void setup()
{
    pinMode(8,  OUTPUT);   //LLED
    pinMode(9,  OUTPUT);   //RLED
    pinMode(10, OUTPUT); //Buzzer
    pinMode(btnCalPin, INPUT);    //Calibrate button
    pinMode(btnBTOffPin, INPUT);    //BT off button
    pinMode(btnBTOffSig, OUTPUT);   //BT OFF signal 
    
    Wire.begin();
  // COMMENT NEXT LINE IF YOU ARE USING ARDUINO DUE
    TWBR = 24; // 400kHz I2C clock (200kHz if CPU is 8MHz). Leonardo measured 250kHz.

    Serial.begin(38400);
    //initialize device
    accelgyro.initialize();
    Serial.println("\nMPU6050 Connection...");
    delay(2000);
    if (accelgyro.testConnection()) {
        Serial.println("MPU6050 connection successful");
    }
    else {
        Serial.println("MPU6050 connection failed");
        while (1) ;
    }
    delay(1000);
    EEPROM.get(0, gv);
    if (gv.ax != 65535) accelgyro.setXAccelOffset(gv.ax);
    else accelgyro.setXAccelOffset(-10);
    if (gv.ay != 65535) accelgyro.setYAccelOffset(gv.ay);
    else accelgyro.setYAccelOffset(-1);
    if (gv.az != 65535) accelgyro.setZAccelOffset(gv.az);
    else accelgyro.setZAccelOffset(16385);
    if (gv.gx != 65535) accelgyro.setXGyroOffset(gv.gx);
    else accelgyro.setXGyroOffset(0);
    if (gv.gy != 65535) accelgyro.setYGyroOffset(gv.gy);
    else accelgyro.setYGyroOffset(0);
    if (gv.gz != 65535) accelgyro.setZGyroOffset(gv.gz);
    else accelgyro.setZGyroOffset(1);
    if (gv.btoff != 255) {
        digitalWrite(btnBTOffSig, HIGH);
        blueToothOff = 1;
        delay(1000);
    }
    else {
        digitalWrite(btnBTOffSig, LOW);
        blueToothOff = 0;
        delay(1000);
        Serial1.begin(btSpeed);
    }
    delay(1000);
    Serial.println("Start reading sensor...");
}

void loop() {
    currTime = millis();
    ll.Preveri(currTime);
    lr.Preveri(currTime);
    btnCalState = digitalRead(btnCalPin);
    
    if (btnCalState == HIGH) {
        tone(buzPin, 2500);
        delay (500);
        noTone(buzPin);
        delay (200);
        tone(buzPin, 2500);
        delay (500);
        noTone(buzPin);
        delay (2000);
        
        calibration();
        
        for (int i = 0; i<3; i++) {
            tone(buzPin, 2500);
            delay (200);
            noTone(buzPin);
            delay (200);
        }        
        tone(buzPin, 2500);
        delay (500);
        noTone(buzPin);
        delay(5000);
    }

    accelgyro.getMotion6(&gv.ax, &gv.ay, &gv.az, &gv.gx, &gv.gy, &gv.gz);

    Serial.print(currTime, DEC);
    Serial.print("\t");
    Serial.print(gv.gy, DEC);
    Serial.print("\t");
    Serial.println(gv.gz, DEC);
    if (blueToothOff == 0) {
        Serial1.print(currTime, DEC);
        Serial1.print("\t");
        Serial1.print(gv.gx, DEC);
        Serial1.print("\t");
        Serial1.print(gv.gy, DEC);
        Serial1.print("\t");
        Serial1.print(gv.gz, DEC);
        Serial1.println("#");
    }
    gyValUD  = gv.gy;
    gyValLR = gv.gz;

    if (gyValUD > MIN_d && ledState != OFF_d)
        gyValUD = OFF_d;

    if (ledState == OFF_d || numSigLR == 0){
        if (gyValLR >= MAX_d) {
            gyValLR = LEFT_d;
            ledState = LEFT_d;
            if (blueToothOff == 0) {
                Serial1.println("Start LEFT LEDS...");
            }
        }
        else {
            if (gyValLR <= MIN_d) {
                gyValLR = RIGHT_d;
                ledState = RIGHT_d;
                if (blueToothOff == 0) {
                    Serial1.println("Start RIGHT LEDS...");
                }
            }
            else {
                gyValLR = OFF_d;
            }
        }
    }
    else {
        if (ledState == LEFT_d) {
            if (gyValLR >= MAX_d) {
                gyValLR = LEFT_d;
            }
            else {
                gyValLR = OFF_d;
            }
        }
        else {
            if (gyValLR <= MIN_d) {
                gyValLR = RIGHT_d;
            }
            else {
                gyValLR = OFF_d;
            }
        }
    }

    if ((numSigUD != 0 || gyValUD != OFF_d) && ledState != OFF_d){
        if (gyValUD != OFF_d){
            if (signalUD == OFF_d){
                startTime = currTime;
            }
            else {
                numSigUD++;
                signalUD = LEFT_d;
            }
        }
        else{
            if ((currTime - startTime) <= MAXSIGTIME_d) {
                if (blueToothOff == 0) {
                    Serial1.println("All LEDS off");
                }
                lr.Off();
                ll.Off();
                ledState = OFF_d;
                oledState = ledState;
                prevSignalLR = OFF_d;
            }
            numSigLR = 0;
            numSigUD = 0;
            signalUD = OFF_;
        }
    }
    else if (numSigLR != 0 || gyValLR != OFF_d) {
        if (numSigLR == 0 ){
            prevSignalLR = gyValLR;
            intTime=currTime;
            sigTime=currTime;
            numSigLR++;
        }
        else{
            if (prevSignalLR != gyValLR) {
                if (gyValLR == OFF_d) {
                    if ((currTime - sigTime) > MAXSIGTIME_d ) {
                        if (numSigLR > 0){
                            numSigLR--;
                        }
                        if (numSigLR == 0){
                            ledState = OFF_d;
                        }
                    }
                }
                else{
                    numSigLR++;
                }
                sigTime=currTime;
                prevSignalLR = gyValLR;
            }
            else{
                if (gyValLR == OFF_d){ 
                    // dva signala: off in nasprotni signal
                    if ((currTime - sigTime) > (2 * MAXSIGTIME_d)){  
                        if ((currTime - intTime) <= MAXINTTIME_d){
                            if (numSigLR == 2){
                                if (ledState != oledState && oledState != OFF_d) {
                                    lr.Off();
                                    ll.Off();
                                }
                                if (ledState == LEFT_d) {
                                    if (blueToothOff == 0) {
                                        Serial1.println("Left Leds ON");
                                    }
                                    ll.OnOff(currTime);
                                }
                                else {
                                    if (blueToothOff == 0) {
                                        Serial1.println("Right Leds ON");
                                    }
                                    lr.OnOff(currTime);
                                }
                                oledState = ledState;
                            }
                        }
                        numSigLR = 0;
                        ledState = OFF_d;
                    }
                }
            }
        }
    }


//    if (led == OFF_d){
//        if (gyValLR >= MAX_d)
//            gyValLR = LEFT_d;
//        else if (gyValLR <= MIN_d)
//            gyValLR = RIGHT_d;
//        else
//            gyValLR = OFF_d;
//    }
//    else if (led == LEFT_d){
//        if (gyValLR >= MAX_d)
//            gyValLR = LEFT_d;
//        else
//            gyValLR = OFF_d;
//    }
//    else {  // RIGHT_d
//        if (gyValLR <= MIN_d)
//            gyValLR = RIGHT_d;
//        else
//            gyValLR = OFF_d;
//    }
//
//    if (numSigUD != 0 || gyValUD != OFF_d) {
//        if (gyValUD != OFF_d) {
//            if (signalUD == OFF_d) {
//                startTime = currTime;
//            }
//            if (signalUD != LEFT_d) {
//              numSigUD++;
//              signalUD = LEFT_d;
//            }
//        }
//        else {
//            if ((currTime - startTime) <= MAXSIGTIME_d) {
//                Serial.println("All LEDS off");
//                lr.Off();
//                ll.Off();
//                ledState = OFF_d;
//                oledState = ledState;
//                led = OFF_d;
//                stateLR  = 0;
//                prevSignalLR = OFF_d;
//            }
//            numSigUD = 0;
//            signalUD = OFF_d;
//        }
//    }
//    else if (stateLR != 0 || gyValLR != OFF_d) {
//        if (gyValLR == LEFT_d) {
//            Serial.println("LEFT_d");
//            checkSignal(LEFT_d);
//        }
//        else if (gyValLR == RIGHT_d) {
//            Serial.println("RIGHT_d");
//            checkSignal(RIGHT_d);
//        }
//        else {   // OFF_d
//            if (prevSignalLR == OFF_d) {
//                if ((currTime - startTime) >= MAXINTTIME_d && stateLR > 0) {
//                    stateLR  = 0;
//                    prevSignalLR = OFF_d;
//                    led = OFF_d;
//                }
//            }
//            else {
//                Serial.println("End of RIGHT_d/LEFT_d");
//                if ((currTime - startTime) >= MAXINTTIME_d) {
//                    stateLR  = 0;
//                    prevSignalLR = OFF_d;
//                    led = OFF_d;
//                }
//                else {
//                    if (stateLR == LAST_d) {
//                        if (ledState != oledState && oledState != OFF_d) {
//                            lr.Off();
//                            ll.Off();
//                        }
//                        if (ledState == LEFT_d) {
//                            ll.OnOff(currTime);
//                        }
//                        else {
//                            lr.OnOff(currTime);
//                        }
//                        oledState = ledState;
//                        stateLR  = 0;
//                        prevSignalLR = OFF_d;
//                        led = OFF_d;
//                    }
//                    else {
//                        stateLR++;
//                        prevSignalLR = OFF_d;
//                    }
//                }
//            }
//        }
//    }
}


void meansensors(){
  long i=0,buff_ax=0,buff_ay=0,buff_az=0,buff_gx=0,buff_gy=0,buff_gz=0;
  int mean_ax,mean_ay,mean_az,mean_gx,mean_gy,mean_gz,state=0;
  for (i = 0; i<(sampleTimes+sampleOffset); i++){
    // read raw accel/gyro measurements from device
    accelgyro.getMotion6(&gv.ax, &gv.ay, &gv.az, &gv.gx, &gv.gy, &gv.gz);
    //In case of calibration first 100 measures are discarded
    if ((i > sampleOffset && i < (sampleTimes + sampleOffset) && state <=1) || state > 1 ){
      buff_ax = buff_ax + gv.ax;
      buff_ay = buff_ay + gv.ay;
      buff_az = buff_az + gv.az;
      buff_gx = buff_gx + gv.gx;
      buff_gy = buff_gy + gv.gy;
      buff_gz = buff_gz + gv.gz;
    }
    delay(2); //Needed so we don't get repeated measures
  }
  mean_ax = buff_ax / sampleTimes;
  mean_ay = buff_ay / sampleTimes;
  mean_az = buff_az / sampleTimes;
  mean_gx = buff_gx / sampleTimes;
  mean_gy = buff_gy / sampleTimes;
  mean_gz = buff_gz / sampleTimes;
}
void calibration(){
  int ax_offset,ay_offset,az_offset,gx_offset,gy_offset,gz_offset;
  int mean_ax,mean_ay,mean_az,mean_gx,mean_gy,mean_gz,state=0;
  ax_offset=-mean_ax/8;
  ay_offset=-mean_ay/8;
  az_offset=(16384-mean_az)/8;
  gx_offset=-mean_gx/4;
  gy_offset=-mean_gy/4;
  gz_offset=-mean_gz/4;
  while (1){
    int ready=0;
    accelgyro.setXAccelOffset(ax_offset);
    accelgyro.setYAccelOffset(ay_offset);
    accelgyro.setZAccelOffset(az_offset);
    accelgyro.setXGyroOffset(gx_offset);
    accelgyro.setYGyroOffset(gy_offset);
    accelgyro.setZGyroOffset(gz_offset);
    meansensors();
    Serial.println("...");
    if (abs(mean_ax)<=acel_deadzone) ready++;
    else ax_offset=ax_offset-mean_ax/acel_deadzone;
    if (abs(mean_ay)<=acel_deadzone) ready++;
    else ay_offset=ay_offset-mean_ay/acel_deadzone;
    if (abs(16384-mean_az)<=acel_deadzone) ready++;
    else az_offset=az_offset+(16384-mean_az)/acel_deadzone;
    if (abs(mean_gx)<=giro_deadzone) ready++;
    else gx_offset=gx_offset-mean_gx/(giro_deadzone+1);
    if (abs(mean_gy)<=giro_deadzone) ready++;
    else gy_offset=gy_offset-mean_gy/(giro_deadzone+1);
    if (abs(mean_gz)<=giro_deadzone) ready++;
    else gz_offset=gz_offset-mean_gz/(giro_deadzone+1);
    if (ready==6) break;
  }
  gv.ax = ax_offset;
  gv.ay = ay_offset;
  gv.az = az_offset;
  gv.gx = gx_offset;
  gv.gy = gy_offset;
  gv.gz = gz_offset;
  gv.btoff = blueToothOff;
  EEPROM.put(0, gv);
}
